import { api } from "~/src/scripts/APILibrary";

export default defineNuxtRouteMiddleware(async (from) => {
  const cookieToken = useCookie("token");

  // If the user is not logged in, take them to the /login page.
  if (!cookieToken.value && from.path !== "/login") {
    return navigateTo("/login");
  }

  // Validate if the user had a valid login token.
  api.addToken(cookieToken.value ?? "");
  const result = await api.get("auth_whoami");

  if (Object.keys(result).length === 0) {
    cookieToken.value = "";
    navigateTo("/login");
    return;
  }

  // If the user is logged in and trying to access /login, direct them to /dashboard.
  if (from.path === "/login" || from.path === "/") {
    return navigateTo("/dashboard");
  }
});
